with Ada.Text_IO; --  to get visibility of Put_Line
with Controller;
with Results;
with Common;
with Calendar;

--  the Main unit is the only thread of control in the whole system
procedure Main is
   use Ada.Text_IO;
   use Common;
   use Calendar;

   Divider : Natural;
   Where   : Common.Partition_ID;
   Starter : Controller.Pool_Access;
   Time_Taken : Duration;
   Start_Time : constant Time := Clock;

begin

   Put_Line ("Program started ...");

   Starter := Controller.Next (Main'Partition_ID);

   for Number in Common.Search_Range loop
      --  RPC directed to the partition designated by the controlling parameter
      Common.Sieve (Starter, Number);
      --  blocking call to a protected entry that returns the divider of Number
      --  and the partition on which that resides
      Results.Load (Divider, Where);
      if Divider = Number then
         Put_Line
           (">>>>>> " & Natural'Image (Number) &
            " (prime located by partition " &
            Partition_ID'Image (Where) & ")");
      else
         Put_Line
           (Natural'Image (Number) &
            "             (divided by" &
            Natural'Image (Divider) &
            " on partition " &
            Partition_ID'Image (Where) & ")");
      end if;
   end loop;

   Time_Taken := Clock - Start_Time;
   Put_Line ("Sieve was called " & Natural'Image (Results.Total) & " times.");
   Put_Line ("Finished. Main ends after " & Time_Taken'Img & " seconds.");

end Main;
